@@ -9,28 +9,45 @@
 #include <algorithm>
 #include <limits>
 #include <memory>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
 #include "core/fpdfapi/page/cpdf_colorspace.h"
 #include "core/fxcodec/codec/cjpx_decoder.h"
-#include "core/fxcrt/fx_memory.h"
 #include "core/fxcrt/fx_safe_types.h"
+#include "third_party/base/optional.h"
 #include "third_party/base/ptr_util.h"
+#include "third_party/base/stl_util.h"
+
+#ifndef USE_SYSTEM_LIBOPENJPEG2
 #include "third_party/libopenjpeg20/openjpeg.h"
 #include "third_party/libopenjpeg20/opj_malloc.h"
+#endif
 
 namespace {
 
+// Used with std::unique_ptr to call opj_image_data_free on raw memory.
+struct OpjImageDataDeleter {
+  inline void operator()(void* ptr) const { opj_image_data_free(ptr); }
+};
+
+using ScopedOpjImageData = std::unique_ptr<int, OpjImageDataDeleter>;
+
+struct OpjImageRgbData {
+  ScopedOpjImageData r;
+  ScopedOpjImageData g;
+  ScopedOpjImageData b;
+};
+
 void fx_ignore_callback(const char* msg, void* client_data) {}
 
-opj_stream_t* fx_opj_stream_create_memory_stream(DecodeData* data,
-                                                 OPJ_SIZE_T p_size,
-                                                 OPJ_BOOL p_is_read_stream) {
+opj_stream_t* fx_opj_stream_create_memory_stream(DecodeData* data) {
   if (!data || !data->src_data || data->src_size <= 0)
     return nullptr;
 
-  opj_stream_t* stream = opj_stream_create(p_size, p_is_read_stream);
+  opj_stream_t* stream = opj_stream_create(OPJ_J2K_STREAM_CHUNK_SIZE,
+                                           /*p_is_input=*/OPJ_TRUE);
   if (!stream)
     return nullptr;
 
@@ -42,28 +59,21 @@ opj_stream_t* fx_opj_stream_create_memory_stream(DecodeData* data,
   return stream;
 }
 
-bool alloc_rgb(int** out_r, int** out_g, int** out_b, size_t size) {
-  int* r = static_cast<int*>(opj_image_data_alloc(size));
-  if (!r)
-    return false;
+Optional<OpjImageRgbData> alloc_rgb(size_t size) {
+  OpjImageRgbData data;
+  data.r.reset(static_cast<int*>(opj_image_data_alloc(size)));
+  if (!data.r)
+    return {};
 
-  int* g = static_cast<int*>(opj_image_data_alloc(size));
-  if (!g) {
-    opj_image_data_free(r);
-    return false;
-  }
+  data.g.reset(static_cast<int*>(opj_image_data_alloc(size)));
+  if (!data.g)
+    return {};
 
-  int* b = static_cast<int*>(opj_image_data_alloc(size));
-  if (!b) {
-    opj_image_data_free(r);
-    opj_image_data_free(g);
-    return false;
-  }
+  data.b.reset(static_cast<int*>(opj_image_data_alloc(size)));
+  if (!data.b)
+    return {};
 
-  *out_r = r;
-  *out_g = g;
-  *out_b = b;
-  return true;
+  return data;
 }
 
 void sycc_to_rgb(int offset,
@@ -104,25 +114,23 @@ void sycc444_to_rgb(opj_image_t* img) {
   if (!y || !cb || !cr)
     return;
 
-  int* r;
-  int* g;
-  int* b;
-  if (!alloc_rgb(&r, &g, &b, max_size.ValueOrDie()))
+  Optional<OpjImageRgbData> data = alloc_rgb(max_size.ValueOrDie());
+  if (!data.has_value())
     return;
 
-  int* d0 = r;
-  int* d1 = g;
-  int* d2 = b;
+  int* r = data.value().r.get();
+  int* g = data.value().g.get();
+  int* b = data.value().b.get();
   max_size /= sizeof(int);
-  for (size_t i = 0; i < max_size.ValueOrDie(); ++i) {
+  for (size_t i = 0; i < max_size.ValueOrDie(); ++i)
     sycc_to_rgb(offset, upb, *y++, *cb++, *cr++, r++, g++, b++);
-  }
+
   opj_image_data_free(img->comps[0].data);
   opj_image_data_free(img->comps[1].data);
   opj_image_data_free(img->comps[2].data);
-  img->comps[0].data = d0;
-  img->comps[1].data = d1;
-  img->comps[2].data = d2;
+  img->comps[0].data = data.value().r.release();
+  img->comps[1].data = data.value().g.release();
+  img->comps[2].data = data.value().b.release();
 }
 
 bool sycc420_422_size_is_valid(opj_image_t* img) {
@@ -166,15 +174,13 @@ void sycc422_to_rgb(opj_image_t* img) {
   if (!y || !cb || !cr)
     return;
 
-  int* r;
-  int* g;
-  int* b;
-  if (!alloc_rgb(&r, &g, &b, max_size.ValueOrDie()))
+  Optional<OpjImageRgbData> data = alloc_rgb(max_size.ValueOrDie());
+  if (!data.has_value())
     return;
 
-  int* d0 = r;
-  int* d1 = g;
-  int* d2 = b;
+  int* r = data.value().r.get();
+  int* g = data.value().g.get();
+  int* b = data.value().b.get();
   for (uint32_t i = 0; i < maxh; ++i) {
     OPJ_UINT32 j;
     for (j = 0; j < (maxw & ~static_cast<OPJ_UINT32>(1)); j += 2) {
@@ -185,12 +191,13 @@ void sycc422_to_rgb(opj_image_t* img) {
       sycc_to_rgb(offset, upb, *y++, *cb++, *cr++, r++, g++, b++);
     }
   }
+
   opj_image_data_free(img->comps[0].data);
   opj_image_data_free(img->comps[1].data);
   opj_image_data_free(img->comps[2].data);
-  img->comps[0].data = d0;
-  img->comps[1].data = d1;
-  img->comps[2].data = d2;
+  img->comps[0].data = data.value().r.release();
+  img->comps[1].data = data.value().g.release();
+  img->comps[2].data = data.value().b.release();
   img->comps[1].w = maxw;
   img->comps[1].h = maxh;
   img->comps[2].w = maxw;
@@ -272,22 +279,22 @@ OPJ_OFF_T opj_skip_from_memory(OPJ_OFF_T nb_bytes, void* p_user_data) {
   if (nb_bytes < 0)
     return static_cast<OPJ_OFF_T>(-1);
 
-  // FIXME: use std::make_unsigned<OPJ_OFF_T>::type once c++11 lib is OK'd.
-  uint64_t unsignedNbBytes = static_cast<uint64_t>(nb_bytes);
+  auto unsigned_nb_bytes =
+      static_cast<std::make_unsigned<OPJ_OFF_T>::type>(nb_bytes);
   // Additionally, the offset may take us beyond the range of a size_t (e.g.
   // 32-bit platforms). If so, just clamp at EOF.
-  if (unsignedNbBytes >
+  if (unsigned_nb_bytes >
       std::numeric_limits<OPJ_SIZE_T>::max() - srcData->offset) {
     srcData->offset = srcData->src_size;
   } else {
-    OPJ_SIZE_T checkedNbBytes = static_cast<OPJ_SIZE_T>(unsignedNbBytes);
+    OPJ_SIZE_T checked_nb_bytes = static_cast<OPJ_SIZE_T>(unsigned_nb_bytes);
     // Otherwise, mimic fseek() semantics to always succeed, even past EOF,
     // clamping at EOF.  We can get away with this since we don't actually
     // provide negative relative skips from beyond EOF back to inside the
     // data, which would be the only reason to need to know exactly how far
     // beyond EOF we are.
     srcData->offset =
-        std::min(srcData->offset + checkedNbBytes, srcData->src_size);
+        std::min(srcData->offset + checked_nb_bytes, srcData->src_size);
   }
   return nb_bytes;
 }
@@ -302,17 +309,17 @@ OPJ_BOOL opj_seek_from_memory(OPJ_OFF_T nb_bytes, void* p_user_data) {
   if (nb_bytes < 0)
     return OPJ_FALSE;
 
-  // FIXME: use std::make_unsigned<OPJ_OFF_T>::type once c++11 lib is OK'd.
-  uint64_t unsignedNbBytes = static_cast<uint64_t>(nb_bytes);
+  auto unsigned_nb_bytes =
+      static_cast<std::make_unsigned<OPJ_OFF_T>::type>(nb_bytes);
   // Additionally, the offset may take us beyond the range of a size_t (e.g.
   // 32-bit platforms). If so, just clamp at EOF.
-  if (unsignedNbBytes > std::numeric_limits<OPJ_SIZE_T>::max()) {
+  if (unsigned_nb_bytes > std::numeric_limits<OPJ_SIZE_T>::max()) {
     srcData->offset = srcData->src_size;
   } else {
-    OPJ_SIZE_T checkedNbBytes = static_cast<OPJ_SIZE_T>(nb_bytes);
+    OPJ_SIZE_T checked_nb_bytes = static_cast<OPJ_SIZE_T>(nb_bytes);
     // Otherwise, mimic fseek() semantics to always succeed, even past EOF,
     // again clamping at EOF.
-    srcData->offset = std::min(checkedNbBytes, srcData->src_size);
+    srcData->offset = std::min(checked_nb_bytes, srcData->src_size);
   }
   return OPJ_TRUE;
 }
@@ -334,27 +341,25 @@ void sycc420_to_rgb(opj_image_t* img) {
   OPJ_UINT32 crw = img->comps[2].w;
   bool extw = sycc420_must_extend_cbcr(yw, cbw);
   bool exth = sycc420_must_extend_cbcr(yh, cbh);
-  FX_SAFE_UINT32 safeSize = yw;
-  safeSize *= yh;
-  safeSize *= sizeof(int);
-  if (!safeSize.IsValid())
+  FX_SAFE_UINT32 safe_size = yw;
+  safe_size *= yh;
+  safe_size *= sizeof(int);
+  if (!safe_size.IsValid())
     return;
 
-  int* r;
-  int* g;
-  int* b;
-  if (!alloc_rgb(&r, &g, &b, safeSize.ValueOrDie()))
-    return;
-
-  int* d0 = r;
-  int* d1 = g;
-  int* d2 = b;
   const int* y = img->comps[0].data;
   const int* cb = img->comps[1].data;
   const int* cr = img->comps[2].data;
   if (!y || !cb || !cr)
     return;
 
+  Optional<OpjImageRgbData> data = alloc_rgb(safe_size.ValueOrDie());
+  if (!data.has_value())
+    return;
+
+  int* r = data.value().r.get();
+  int* g = data.value().g.get();
+  int* b = data.value().b.get();
   const int* ny = nullptr;
   int* nr = nullptr;
   int* ng = nullptr;
@@ -444,9 +449,9 @@ void sycc420_to_rgb(opj_image_t* img) {
   opj_image_data_free(img->comps[0].data);
   opj_image_data_free(img->comps[1].data);
   opj_image_data_free(img->comps[2].data);
-  img->comps[0].data = d0;
-  img->comps[1].data = d1;
-  img->comps[2].data = d2;
+  img->comps[0].data = data.value().r.release();
+  img->comps[1].data = data.value().g.release();
+  img->comps[2].data = data.value().b.release();
   img->comps[1].w = yw;
   img->comps[1].h = yh;
   img->comps[2].w = yw;
@@ -483,9 +488,7 @@ bool CJPX_Decoder::Init(pdfium::span<const uint8_t> src_data) {
   m_SrcData = src_data;
   m_DecodeData =
       pdfium::MakeUnique<DecodeData>(src_data.data(), src_data.size());
-  m_Stream = fx_opj_stream_create_memory_stream(
-      m_DecodeData.get(), static_cast<unsigned int>(OPJ_J2K_STREAM_CHUNK_SIZE),
-      1);
+  m_Stream = fx_opj_stream_create_memory_stream(m_DecodeData.get());
   if (!m_Stream)
     return false;
 
@@ -515,8 +518,13 @@ bool CJPX_Decoder::Init(pdfium::span<const uint8_t> src_data) {
     return false;
 
   m_Image = pTempImage;
+#ifndef USE_SYSTEM_LIBOPENJPEG2
   m_Image->pdfium_use_colorspace = !!m_ColorSpace;
+#endif
+  return true;
+}
 
+bool CJPX_Decoder::StartDecode() {
   if (!m_Parameters.nb_tile_to_decode) {
     if (!opj_set_decode_area(m_Codec.Get(), m_Image.Get(), m_Parameters.DA_x0,
                              m_Parameters.DA_y0, m_Parameters.DA_x1,
@@ -549,11 +557,14 @@ bool CJPX_Decoder::Init(pdfium::span<const uint8_t> src_data) {
     // TODO(palmer): Using |opj_free| here resolves the crash described in
     // https://crbug.com/737033, but ultimately we need to harmonize the
     // memory allocation strategy across OpenJPEG and its PDFium callers.
+#ifndef USE_SYSTEM_LIBOPENJPEG2
     opj_free(m_Image->icc_profile_buf);
+#else
+    free(m_Image->icc_profile_buf);
+#endif
     m_Image->icc_profile_buf = nullptr;
     m_Image->icc_profile_len = 0;
   }
-
   return true;
 }
 
