@@ -11,6 +11,8 @@
 #include <utility>
 #include <vector>
 
+#include "build/build_config.h"
+#include "core/fxcodec/codec/cfx_codec_memory.h"
 #include "core/fxcodec/fx_codec.h"
 #include "core/fxcrt/fx_stream.h"
 #include "core/fxge/dib/cfx_dibitmap.h"
@@ -24,11 +26,11 @@
 namespace {
 
 #ifdef PDF_ENABLE_XFA_PNG
-#if _FX_OS_ == _FX_OS_MACOSX_
+#if defined(OS_MACOSX)
 const double kPngGamma = 1.7;
-#else   // _FX_OS_ == _FX_OS_MACOSX_
+#else
 const double kPngGamma = 2.2;
-#endif  // _FX_OS_ == _FX_OS_MACOSX_
+#endif  // defined(OS_MACOSX)
 #endif  // PDF_ENABLE_XFA_PNG
 
 void RGB2BGR(uint8_t* buffer, int width = 1) {
@@ -716,9 +718,9 @@ bool CCodec_ProgressiveDecoder::BmpDetectImageTypeInBuffer(
     return false;
   }
 
-  std::unique_ptr<CCodec_BmpModule::Context> pBmpContext =
+  std::unique_ptr<CodecModuleIface::Context> pBmpContext =
       pBmpModule->Start(this);
-  pBmpModule->Input(pBmpContext.get(), {m_pSrcBuf.get(), m_SrcSize});
+  pBmpModule->Input(pBmpContext.get(), m_pCodecMemory, nullptr);
 
   std::vector<uint32_t> palette;
   int32_t readResult = pBmpModule->ReadHeader(
@@ -767,7 +769,9 @@ bool CCodec_ProgressiveDecoder::BmpDetectImageTypeInBuffer(
     return false;
   }
 
-  uint32_t availableData = m_SrcSize > m_offSet ? m_SrcSize - m_offSet : 0;
+  uint32_t availableData = m_pCodecMemory->GetSize() > m_offSet
+                               ? m_pCodecMemory->GetSize() - m_offSet
+                               : 0;
   if (neededData > availableData) {
     m_status = FXCODEC_STATUS_ERR_FORMAT;
     return false;
@@ -788,45 +792,9 @@ bool CCodec_ProgressiveDecoder::BmpDetectImageTypeInBuffer(
 
 bool CCodec_ProgressiveDecoder::BmpReadMoreData(
     CCodec_BmpModule* pBmpModule,
-    CCodec_BmpModule::Context* pBmpContext,
+    CodecModuleIface::Context* pContext,
     FXCODEC_STATUS& err_status) {
-  uint32_t dwSize = (uint32_t)m_pFile->GetSize();
-  if (dwSize <= m_offSet)
-    return false;
-
-  dwSize = dwSize - m_offSet;
-  FX_SAFE_UINT32 avail_input = pBmpModule->GetAvailInput(pBmpContext);
-  if (!avail_input.IsValid())
-    return false;
-
-  uint32_t dwAvail = avail_input.ValueOrDie();
-  if (dwAvail == m_SrcSize) {
-    if (dwSize > FXCODEC_BLOCK_SIZE) {
-      dwSize = FXCODEC_BLOCK_SIZE;
-    }
-    m_SrcSize = (dwSize + dwAvail + FXCODEC_BLOCK_SIZE - 1) /
-                FXCODEC_BLOCK_SIZE * FXCODEC_BLOCK_SIZE;
-    m_pSrcBuf.reset(FX_TryRealloc(uint8_t, m_pSrcBuf.release(), m_SrcSize));
-    if (!m_pSrcBuf) {
-      err_status = FXCODEC_STATUS_ERR_MEMORY;
-      return false;
-    }
-  } else {
-    uint32_t dwConsume = m_SrcSize - dwAvail;
-    if (dwAvail) {
-      memmove(m_pSrcBuf.get(), m_pSrcBuf.get() + dwConsume, dwAvail);
-    }
-    if (dwSize > dwConsume) {
-      dwSize = dwConsume;
-    }
-  }
-  if (!m_pFile->ReadBlock(m_pSrcBuf.get() + dwAvail, m_offSet, dwSize)) {
-    err_status = FXCODEC_STATUS_ERR_READ;
-    return false;
-  }
-  m_offSet += dwSize;
-  pBmpModule->Input(pBmpContext, {m_pSrcBuf.get(), dwSize + dwAvail});
-  return true;
+  return ReadMoreData(pBmpModule, pContext, false, err_status);
 }
 
 FXCODEC_STATUS CCodec_ProgressiveDecoder::BmpStartDecode(
@@ -839,9 +807,8 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::BmpStartDecode(
     return m_status;
   }
   GetTransMethod(m_pDeviceBitmap->GetFormat(), m_SrcFormat);
-  m_ScanlineSize = (m_SrcWidth * m_SrcComponents + 3) / 4 * 4;
+  m_ScanlineSize = FxAlignToBoundary<4>(m_SrcWidth * m_SrcComponents);
   m_pDecodeBuf.reset(FX_Alloc(uint8_t, m_ScanlineSize));
-  memset(m_pDecodeBuf.get(), 0, m_ScanlineSize);
   m_WeightHorz.Calc(m_sizeX, 0, m_sizeX, m_clipBox.Width(), 0,
                     m_clipBox.Width());
   m_WeightVert.Calc(m_sizeY, m_clipBox.Height());
@@ -884,61 +851,26 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::BmpContinueDecode() {
 #ifdef PDF_ENABLE_XFA_GIF
 bool CCodec_ProgressiveDecoder::GifReadMoreData(CCodec_GifModule* pGifModule,
                                                 FXCODEC_STATUS& err_status) {
-  if (static_cast<uint32_t>(m_pFile->GetSize()) <= m_offSet)
-    return false;
-
-  uint32_t dwFileRemaining = m_pFile->GetSize() - m_offSet;
-  uint32_t dwUnusedBuffer = !m_InvalidateGifBuffer
-                                ? pGifModule->GetAvailInput(m_pGifContext.get())
-                                : 0;
-  uint32_t dwAmountToFetchFromFile = dwFileRemaining;
-  if (dwUnusedBuffer == m_SrcSize) {
-    if (dwFileRemaining > FXCODEC_BLOCK_SIZE)
-      dwAmountToFetchFromFile = FXCODEC_BLOCK_SIZE;
-    m_SrcSize = std::min(
-        (dwAmountToFetchFromFile + dwUnusedBuffer + FXCODEC_BLOCK_SIZE - 1) /
-            FXCODEC_BLOCK_SIZE * FXCODEC_BLOCK_SIZE,
-        static_cast<uint32_t>(m_pFile->GetSize()));
-    m_pSrcBuf.reset(FX_TryRealloc(uint8_t, m_pSrcBuf.release(), m_SrcSize));
-    if (!m_pSrcBuf) {
-      err_status = FXCODEC_STATUS_ERR_MEMORY;
-      return false;
-    }
-  } else {
-    uint32_t dwConsumed = m_SrcSize - dwUnusedBuffer;
-    if (dwUnusedBuffer)
-      memmove(m_pSrcBuf.get(), m_pSrcBuf.get() + dwConsumed, dwUnusedBuffer);
-    if (dwFileRemaining > dwConsumed)
-      dwAmountToFetchFromFile = dwConsumed;
-  }
-
-  if (!m_pFile->ReadBlock(m_pSrcBuf.get() + dwUnusedBuffer, m_offSet,
-                          dwAmountToFetchFromFile)) {
-    err_status = FXCODEC_STATUS_ERR_READ;
+  if (!ReadMoreData(pGifModule, m_pGifContext.get(), m_InvalidateGifBuffer,
+                    err_status)) {
     return false;
   }
-
-  m_offSet += dwAmountToFetchFromFile;
-  pGifModule->Input(
-      m_pGifContext.get(),
-      {m_pSrcBuf.get(), dwAmountToFetchFromFile + dwUnusedBuffer});
   m_InvalidateGifBuffer = false;
   return true;
 }
 
-bool CCodec_ProgressiveDecoder::GifDetectImageTypeInBuffer(
-    CFX_DIBAttribute* pAttribute) {
+bool CCodec_ProgressiveDecoder::GifDetectImageTypeInBuffer() {
   CCodec_GifModule* pGifModule = m_pCodecMgr->GetGifModule();
   if (!pGifModule) {
     m_status = FXCODEC_STATUS_ERR_MEMORY;
     return false;
   }
   m_pGifContext = pGifModule->Start(this);
-  pGifModule->Input(m_pGifContext.get(), {m_pSrcBuf.get(), m_SrcSize});
+  pGifModule->Input(m_pGifContext.get(), m_pCodecMemory, nullptr);
   m_SrcComponents = 1;
-  CFX_GifDecodeStatus readResult = pGifModule->ReadHeader(
-      m_pGifContext.get(), &m_SrcWidth, &m_SrcHeight, &m_GifPltNumber,
-      &m_pGifPalette, &m_GifBgIndex, nullptr);
+  CFX_GifDecodeStatus readResult =
+      pGifModule->ReadHeader(m_pGifContext.get(), &m_SrcWidth, &m_SrcHeight,
+                             &m_GifPltNumber, &m_pGifPalette, &m_GifBgIndex);
   while (readResult == CFX_GifDecodeStatus::Unfinished) {
     FXCODEC_STATUS error_status = FXCODEC_STATUS_ERR_FORMAT;
     if (!GifReadMoreData(pGifModule, error_status)) {
@@ -946,9 +878,9 @@ bool CCodec_ProgressiveDecoder::GifDetectImageTypeInBuffer(
       m_status = error_status;
       return false;
     }
-    readResult = pGifModule->ReadHeader(m_pGifContext.get(), &m_SrcWidth,
-                                        &m_SrcHeight, &m_GifPltNumber,
-                                        &m_pGifPalette, &m_GifBgIndex, nullptr);
+    readResult =
+        pGifModule->ReadHeader(m_pGifContext.get(), &m_SrcWidth, &m_SrcHeight,
+                               &m_GifPltNumber, &m_pGifPalette, &m_GifBgIndex);
   }
   if (readResult == CFX_GifDecodeStatus::Success) {
     m_SrcBPC = 8;
@@ -971,9 +903,8 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::GifStartDecode(
   }
   m_SrcFormat = FXCodec_8bppRgb;
   GetTransMethod(m_pDeviceBitmap->GetFormat(), m_SrcFormat);
-  int scanline_size = (m_SrcWidth + 3) / 4 * 4;
+  int scanline_size = FxAlignToBoundary<4>(m_SrcWidth);
   m_pDecodeBuf.reset(FX_Alloc(uint8_t, scanline_size));
-  memset(m_pDecodeBuf.get(), 0, scanline_size);
   m_WeightHorz.Calc(m_sizeX, 0, m_sizeX, m_clipBox.Width(), 0,
                     m_clipBox.Width());
   m_WeightVert.Calc(m_sizeY, m_clipBox.Height());
@@ -992,7 +923,7 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::GifContinueDecode() {
   }
 
   CFX_GifDecodeStatus readRes =
-      pGifModule->LoadFrame(m_pGifContext.get(), m_FrameCur, nullptr);
+      pGifModule->LoadFrame(m_pGifContext.get(), m_FrameCur);
   while (readRes == CFX_GifDecodeStatus::Unfinished) {
     FXCODEC_STATUS error_status = FXCODEC_STATUS_DECODE_FINISH;
     if (!GifReadMoreData(pGifModule, error_status)) {
@@ -1001,7 +932,7 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::GifContinueDecode() {
       m_status = error_status;
       return m_status;
     }
-    readRes = pGifModule->LoadFrame(m_pGifContext.get(), m_FrameCur, nullptr);
+    readRes = pGifModule->LoadFrame(m_pGifContext.get(), m_FrameCur);
   }
 
   if (readRes == CFX_GifDecodeStatus::Success) {
@@ -1107,39 +1038,7 @@ void CCodec_ProgressiveDecoder::GifDoubleLineResampleVert(
 
 bool CCodec_ProgressiveDecoder::JpegReadMoreData(CCodec_JpegModule* pJpegModule,
                                                  FXCODEC_STATUS& err_status) {
-  uint32_t dwSize = (uint32_t)m_pFile->GetSize();
-  if (dwSize <= m_offSet) {
-    return false;
-  }
-  dwSize = dwSize - m_offSet;
-  uint32_t dwAvail = pJpegModule->GetAvailInput(m_pJpegContext.get());
-  if (dwAvail == m_SrcSize) {
-    if (dwSize > FXCODEC_BLOCK_SIZE) {
-      dwSize = FXCODEC_BLOCK_SIZE;
-    }
-    m_SrcSize = (dwSize + dwAvail + FXCODEC_BLOCK_SIZE - 1) /
-                FXCODEC_BLOCK_SIZE * FXCODEC_BLOCK_SIZE;
-    m_pSrcBuf.reset(FX_TryRealloc(uint8_t, m_pSrcBuf.release(), m_SrcSize));
-    if (!m_pSrcBuf) {
-      err_status = FXCODEC_STATUS_ERR_MEMORY;
-      return false;
-    }
-  } else {
-    uint32_t dwConsume = m_SrcSize - dwAvail;
-    if (dwAvail) {
-      memmove(m_pSrcBuf.get(), m_pSrcBuf.get() + dwConsume, dwAvail);
-    }
-    if (dwSize > dwConsume) {
-      dwSize = dwConsume;
-    }
-  }
-  if (!m_pFile->ReadBlock(m_pSrcBuf.get() + dwAvail, m_offSet, dwSize)) {
-    err_status = FXCODEC_STATUS_ERR_READ;
-    return false;
-  }
-  m_offSet += dwSize;
-  pJpegModule->Input(m_pJpegContext.get(), m_pSrcBuf.get(), dwSize + dwAvail);
-  return true;
+  return ReadMoreData(pJpegModule, m_pJpegContext.get(), false, err_status);
 }
 
 bool CCodec_ProgressiveDecoder::JpegDetectImageTypeInBuffer(
@@ -1150,10 +1049,11 @@ bool CCodec_ProgressiveDecoder::JpegDetectImageTypeInBuffer(
     m_status = FXCODEC_STATUS_ERR_MEMORY;
     return false;
   }
-  pJpegModule->Input(m_pJpegContext.get(), m_pSrcBuf.get(), m_SrcSize);
+  pJpegModule->Input(m_pJpegContext.get(), m_pCodecMemory, nullptr);
+
   // Setting jump marker before calling ReadHeader, since a longjmp to
   // the marker indicates a fatal error.
-  if (setjmp(*m_pJpegContext->GetJumpMark()) == -1) {
+  if (setjmp(*pJpegModule->GetJumpMark(m_pJpegContext.get())) == -1) {
     m_pJpegContext.reset();
     m_status = FXCODEC_STATUS_ERR_FORMAT;
     return false;
@@ -1184,17 +1084,17 @@ bool CCodec_ProgressiveDecoder::JpegDetectImageTypeInBuffer(
 
 FXCODEC_STATUS CCodec_ProgressiveDecoder::JpegStartDecode(
     const RetainPtr<CFX_DIBitmap>& pDIBitmap) {
+  CCodec_JpegModule* pJpegModule = m_pCodecMgr->GetJpegModule();
   int down_scale = 1;
   GetDownScale(down_scale);
   // Setting jump marker before calling StartScanLine, since a longjmp to
   // the marker indicates a fatal error.
-  if (setjmp(*m_pJpegContext->GetJumpMark()) == -1) {
+  if (setjmp(*pJpegModule->GetJumpMark(m_pJpegContext.get())) == -1) {
     m_pJpegContext.reset();
     m_status = FXCODEC_STATUS_ERROR;
     return FXCODEC_STATUS_ERROR;
   }
 
-  CCodec_JpegModule* pJpegModule = m_pCodecMgr->GetJpegModule();
   bool startStatus =
       pJpegModule->StartScanline(m_pJpegContext.get(), down_scale);
   while (!startStatus) {
@@ -1209,9 +1109,8 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::JpegStartDecode(
     startStatus = pJpegModule->StartScanline(m_pJpegContext.get(), down_scale);
   }
   int scanline_size = (m_SrcWidth + down_scale - 1) / down_scale;
-  scanline_size = (scanline_size * m_SrcComponents + 3) / 4 * 4;
+  scanline_size = FxAlignToBoundary<4>(scanline_size * m_SrcComponents);
   m_pDecodeBuf.reset(FX_Alloc(uint8_t, scanline_size));
-  memset(m_pDecodeBuf.get(), 0, scanline_size);
   m_WeightHorz.Calc(m_sizeX, 0, m_sizeX, m_clipBox.Width(), 0,
                     m_clipBox.Width());
   m_WeightVert.Calc(m_sizeY, m_clipBox.Height());
@@ -1235,7 +1134,7 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::JpegContinueDecode() {
   CCodec_JpegModule* pJpegModule = m_pCodecMgr->GetJpegModule();
   // Setting jump marker before calling ReadScanLine, since a longjmp to
   // the marker indicates a fatal error.
-  if (setjmp(*m_pJpegContext->GetJumpMark()) == -1) {
+  if (setjmp(*pJpegModule->GetJumpMark(m_pJpegContext.get())) == -1) {
     m_pJpegContext.reset();
     m_status = FXCODEC_STATUS_ERROR;
     return FXCODEC_STATUS_ERROR;
@@ -1360,9 +1259,7 @@ bool CCodec_ProgressiveDecoder::PngDetectImageTypeInBuffer(
     m_status = FXCODEC_STATUS_ERR_MEMORY;
     return false;
   }
-  bool bResult = pPngModule->Input(m_pPngContext.get(), m_pSrcBuf.get(),
-                                   m_SrcSize, pAttribute);
-  while (bResult) {
+  while (pPngModule->Input(m_pPngContext.get(), m_pCodecMemory, pAttribute)) {
     uint32_t remain_size = static_cast<uint32_t>(m_pFile->GetSize()) - m_offSet;
     uint32_t input_size =
         remain_size > FXCODEC_BLOCK_SIZE ? FXCODEC_BLOCK_SIZE : remain_size;
@@ -1371,19 +1268,15 @@ bool CCodec_ProgressiveDecoder::PngDetectImageTypeInBuffer(
       m_status = FXCODEC_STATUS_ERR_FORMAT;
       return false;
     }
-    if (m_pSrcBuf && input_size > m_SrcSize) {
-      m_pSrcBuf.reset(FX_Alloc(uint8_t, input_size));
-      memset(m_pSrcBuf.get(), 0, input_size);
-      m_SrcSize = input_size;
-    }
-    bResult = m_pFile->ReadBlock(m_pSrcBuf.get(), m_offSet, input_size);
-    if (!bResult) {
+    if (m_pCodecMemory && input_size > m_pCodecMemory->GetSize())
+      m_pCodecMemory = pdfium::MakeRetain<CFX_CodecMemory>(input_size);
+
+    if (!m_pFile->ReadBlockAtOffset(m_pCodecMemory->GetBuffer(), m_offSet,
+                                    input_size)) {
       m_status = FXCODEC_STATUS_ERR_READ;
       return false;
     }
     m_offSet += input_size;
-    bResult = pPngModule->Input(m_pPngContext.get(), m_pSrcBuf.get(),
-                                input_size, pAttribute);
   }
   m_pPngContext.reset();
   if (m_SrcPassNumber == 0) {
@@ -1433,9 +1326,8 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::PngStartDecode(
     }
   }
   GetTransMethod(m_pDeviceBitmap->GetFormat(), m_SrcFormat);
-  int scanline_size = (m_SrcWidth * m_SrcComponents + 3) / 4 * 4;
+  int scanline_size = FxAlignToBoundary<4>(m_SrcWidth * m_SrcComponents);
   m_pDecodeBuf.reset(FX_Alloc(uint8_t, scanline_size));
-  memset(m_pDecodeBuf.get(), 0, scanline_size);
   m_WeightHorzOO.Calc(m_sizeX, m_clipBox.Width());
   m_WeightVert.Calc(m_sizeY, m_clipBox.Height());
   m_status = FXCODEC_STATUS_DECODE_TOBECONTINUE;
@@ -1459,12 +1351,11 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::PngContinueDecode() {
       m_status = FXCODEC_STATUS_DECODE_FINISH;
       return m_status;
     }
-    if (m_pSrcBuf && input_size > m_SrcSize) {
-      m_pSrcBuf.reset(FX_Alloc(uint8_t, input_size));
-      memset(m_pSrcBuf.get(), 0, input_size);
-      m_SrcSize = input_size;
-    }
-    bool bResult = m_pFile->ReadBlock(m_pSrcBuf.get(), m_offSet, input_size);
+    if (m_pCodecMemory && input_size > m_pCodecMemory->GetSize())
+      m_pCodecMemory = pdfium::MakeRetain<CFX_CodecMemory>(input_size);
+
+    bool bResult = m_pFile->ReadBlockAtOffset(m_pCodecMemory->GetBuffer(),
+                                              m_offSet, input_size);
     if (!bResult) {
       m_pDeviceBitmap = nullptr;
       m_pFile = nullptr;
@@ -1472,8 +1363,7 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::PngContinueDecode() {
       return m_status;
     }
     m_offSet += input_size;
-    bResult = pPngModule->Input(m_pPngContext.get(), m_pSrcBuf.get(),
-                                input_size, nullptr);
+    bResult = pPngModule->Input(m_pPngContext.get(), m_pCodecMemory, nullptr);
     if (!bResult) {
       m_pDeviceBitmap = nullptr;
       m_pFile = nullptr;
@@ -1633,8 +1523,8 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::TiffContinueDecode() {
     m_status = FXCODEC_STATUS_ERR_MEMORY;
     return m_status;
   }
-  RetainPtr<CFX_DIBitmap> pStrechBitmap =
-      pFormatBitmap->StretchTo(m_sizeX, m_sizeY, FXDIB_INTERPOL, nullptr);
+  RetainPtr<CFX_DIBitmap> pStrechBitmap = pFormatBitmap->StretchTo(
+      m_sizeX, m_sizeY, kBilinearInterpolation, nullptr);
   pFormatBitmap = nullptr;
   if (!pStrechBitmap) {
     m_pDeviceBitmap = nullptr;
@@ -1659,15 +1549,14 @@ bool CCodec_ProgressiveDecoder::DetectImageType(FXCODEC_IMAGE_TYPE imageType,
 #endif  // PDF_ENABLE_XFA_TIFF
 
   size_t size = std::min<size_t>(m_pFile->GetSize(), FXCODEC_BLOCK_SIZE);
-  m_SrcSize = static_cast<uint32_t>(size);
-  m_pSrcBuf.reset(FX_Alloc(uint8_t, m_SrcSize));
-
+  m_pCodecMemory = pdfium::MakeRetain<CFX_CodecMemory>(size);
   m_offSet = 0;
-  if (!m_pFile->ReadBlock(m_pSrcBuf.get(), m_offSet, m_SrcSize)) {
+  if (!m_pFile->ReadBlockAtOffset(m_pCodecMemory->GetBuffer(), m_offSet,
+                                  size)) {
     m_status = FXCODEC_STATUS_ERR_READ;
     return false;
   }
-  m_offSet += m_SrcSize;
+  m_offSet += size;
 
   if (imageType == FXCODEC_IMAGE_JPG)
     return JpegDetectImageTypeInBuffer(pAttribute);
@@ -1679,7 +1568,7 @@ bool CCodec_ProgressiveDecoder::DetectImageType(FXCODEC_IMAGE_TYPE imageType,
 
 #ifdef PDF_ENABLE_XFA_GIF
   if (imageType == FXCODEC_IMAGE_GIF)
-    return GifDetectImageTypeInBuffer(pAttribute);
+    return GifDetectImageTypeInBuffer();
 #endif  // PDF_ENABLE_XFA_GIF
 
 #ifdef PDF_ENABLE_XFA_PNG
@@ -1691,11 +1580,62 @@ bool CCodec_ProgressiveDecoder::DetectImageType(FXCODEC_IMAGE_TYPE imageType,
   return false;
 }
 
+bool CCodec_ProgressiveDecoder::ReadMoreData(
+    CodecModuleIface* pModule,
+    CodecModuleIface::Context* pContext,
+    bool invalidate_buffer,
+    FXCODEC_STATUS& err_status) {
+  // Check for EOF.
+  if (m_offSet >= static_cast<uint32_t>(m_pFile->GetSize()))
+    return false;
+
+  // Try to get whatever remains.
+  uint32_t dwBytesToFetchFromFile = m_pFile->GetSize() - m_offSet;
+
+  // Figure out if the codec stopped processing midway through the buffer.
+  size_t dwUnconsumed = 0;
+  if (!invalidate_buffer) {
+    FX_SAFE_SIZE_T avail_input = pModule->GetAvailInput(pContext);
+    if (!avail_input.IsValid())
+      return false;
+    dwUnconsumed = avail_input.ValueOrDie();
+  }
+
+  if (dwUnconsumed == m_pCodecMemory->GetSize()) {
+    // Codec couldn't make any progress against the bytes in the buffer.
+    // Increase the buffer size so that there might be enough contiguous
+    // bytes to allow whatever operation is having difficulty to succeed.
+    dwBytesToFetchFromFile =
+        std::min<uint32_t>(dwBytesToFetchFromFile, FXCODEC_BLOCK_SIZE);
+    size_t dwNewSize = m_pCodecMemory->GetSize() + dwBytesToFetchFromFile;
+    if (!m_pCodecMemory->TryResize(dwNewSize)) {
+      err_status = FXCODEC_STATUS_ERR_MEMORY;
+      return false;
+    }
+  } else {
+    size_t dwConsumed = m_pCodecMemory->GetSize() - dwUnconsumed;
+    m_pCodecMemory->Consume(dwConsumed);
+    dwBytesToFetchFromFile =
+        std::min<uint32_t>(dwBytesToFetchFromFile, dwConsumed);
+  }
+
+  // Append new data past the bytes not yet processed by the codec.
+  if (!m_pFile->ReadBlockAtOffset(m_pCodecMemory->GetBuffer() + dwUnconsumed,
+                                  m_offSet, dwBytesToFetchFromFile)) {
+    err_status = FXCODEC_STATUS_ERR_READ;
+    return false;
+  }
+  m_offSet += dwBytesToFetchFromFile;
+  return pModule->Input(pContext, m_pCodecMemory, nullptr);
+}
+
 FXCODEC_STATUS CCodec_ProgressiveDecoder::LoadImageInfo(
     const RetainPtr<IFX_SeekableReadStream>& pFile,
     FXCODEC_IMAGE_TYPE imageType,
     CFX_DIBAttribute* pAttribute,
     bool bSkipImageTypeCheck) {
+  ASSERT(pAttribute);
+
   switch (m_status) {
     case FXCODEC_STATUS_FRAME_READY:
     case FXCODEC_STATUS_FRAME_TOBECONTINUE:
@@ -1714,7 +1654,7 @@ FXCODEC_STATUS CCodec_ProgressiveDecoder::LoadImageInfo(
   m_offSet = 0;
   m_SrcWidth = m_SrcHeight = 0;
   m_SrcComponents = m_SrcBPC = 0;
-  m_clipBox = FX_RECT(0, 0, 0, 0);
+  m_clipBox = FX_RECT();
   m_startX = m_startY = 0;
   m_sizeX = m_sizeY = 0;
   m_SrcPassNumber = 0;
@@ -1747,7 +1687,7 @@ void CCodec_ProgressiveDecoder::SetClipBox(FX_RECT* clip) {
     return;
 
   if (clip->IsEmpty()) {
-    m_clipBox = FX_RECT(0, 0, 0, 0);
+    m_clipBox = FX_RECT();
     return;
   }
   clip->left = std::max(clip->left, 0);
@@ -1755,7 +1695,7 @@ void CCodec_ProgressiveDecoder::SetClipBox(FX_RECT* clip) {
   clip->top = std::max(clip->top, 0);
   clip->bottom = std::min(clip->bottom, m_SrcHeight);
   if (clip->IsEmpty()) {
-    m_clipBox = FX_RECT(0, 0, 0, 0);
+    m_clipBox = FX_RECT();
     return;
   }
   m_clipBox = *clip;
