@@ -5,10 +5,14 @@
 #include "core/fxcrt/widestring.h"
 
 #include <algorithm>
+#include <iterator>
 #include <vector>
 
+#include "build/build_config.h"
 #include "core/fxcrt/fx_string.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "third_party/base/span.h"
+#include "third_party/base/stl_util.h"
 
 namespace fxcrt {
 
@@ -21,6 +25,10 @@ TEST(WideString, ElementAccess) {
   EXPECT_DEATH({ abc[4]; }, ".*");
 #endif
 
+  pdfium::span<const wchar_t> abc_span = abc.AsSpan();
+  EXPECT_EQ(3u, abc_span.size());
+  EXPECT_EQ(0, wmemcmp(abc_span.data(), L"abc", 3));
+
   WideString mutable_abc = abc;
   EXPECT_EQ(abc.c_str(), mutable_abc.c_str());
   EXPECT_EQ(L'a', mutable_abc[0]);
@@ -49,34 +57,141 @@ TEST(WideString, ElementAccess) {
 #endif
 }
 
+TEST(WideString, Assign) {
+  {
+    // Copy-assign.
+    WideString string1;
+    EXPECT_EQ(0, string1.ReferenceCountForTesting());
+    {
+      WideString string2(L"abc");
+      EXPECT_EQ(1, string2.ReferenceCountForTesting());
+
+      string1 = string2;
+      EXPECT_EQ(2, string1.ReferenceCountForTesting());
+      EXPECT_EQ(2, string2.ReferenceCountForTesting());
+    }
+    EXPECT_EQ(1, string1.ReferenceCountForTesting());
+  }
+  {
+    // Move-assign.
+    WideString string1;
+    EXPECT_EQ(0, string1.ReferenceCountForTesting());
+    {
+      WideString string2(L"abc");
+      EXPECT_EQ(1, string2.ReferenceCountForTesting());
+
+      string1 = std::move(string2);
+      EXPECT_EQ(1, string1.ReferenceCountForTesting());
+      EXPECT_EQ(0, string2.ReferenceCountForTesting());
+    }
+    EXPECT_EQ(1, string1.ReferenceCountForTesting());
+  }
+}
+
 TEST(WideString, OperatorLT) {
   WideString empty;
   WideString a(L"a");
+  WideString ab(L"ab");
   WideString abc(L"\x0110qq");  // Comes before despite endianness.
   WideString def(L"\x1001qq");  // Comes after despite endianness.
+  WideStringView v_empty;
+  WideStringView v_a(L"a");
+  WideStringView v_ab(L"ab");
+  WideStringView v_abc(L"\x0110qq");
+  WideStringView v_def(L"\x1001qq");
+  const wchar_t* const c_null = nullptr;
+  const wchar_t* const c_empty = L"";
+  const wchar_t* const c_a = L"a";
+  const wchar_t* const c_ab = L"ab";
+  const wchar_t* const c_abc = L"\x0110qq";
+  const wchar_t* const c_def = L"\x1001qq";
 
   EXPECT_FALSE(empty < empty);
   EXPECT_FALSE(a < a);
   EXPECT_FALSE(abc < abc);
   EXPECT_FALSE(def < def);
+  EXPECT_FALSE(c_null < empty);
+  EXPECT_FALSE(c_empty < empty);
+  EXPECT_FALSE(c_a < a);
+  EXPECT_FALSE(c_abc < abc);
+  EXPECT_FALSE(c_def < def);
+  EXPECT_FALSE(empty < c_null);
+  EXPECT_FALSE(empty < c_empty);
+  EXPECT_FALSE(a < c_a);
+  EXPECT_FALSE(abc < c_abc);
+  EXPECT_FALSE(def < c_def);
+  EXPECT_FALSE(empty < v_empty);
+  EXPECT_FALSE(a < v_a);
+  EXPECT_FALSE(abc < v_abc);
+  EXPECT_FALSE(def < v_def);
 
   EXPECT_TRUE(empty < a);
   EXPECT_FALSE(a < empty);
+  EXPECT_TRUE(c_null < a);
+  EXPECT_TRUE(c_empty < a);
+  EXPECT_FALSE(c_a < empty);
+  EXPECT_TRUE(empty < c_a);
+  EXPECT_FALSE(a < c_null);
+  EXPECT_FALSE(a < c_empty);
+  EXPECT_TRUE(empty < v_a);
+  EXPECT_FALSE(a < v_empty);
 
   EXPECT_TRUE(empty < abc);
   EXPECT_FALSE(abc < empty);
+  EXPECT_TRUE(c_null < abc);
+  EXPECT_TRUE(c_empty < abc);
+  EXPECT_FALSE(c_abc < empty);
+  EXPECT_TRUE(empty < c_abc);
+  EXPECT_FALSE(abc < c_null);
+  EXPECT_FALSE(abc < c_empty);
+  EXPECT_TRUE(empty < v_abc);
+  EXPECT_FALSE(abc < v_empty);
 
   EXPECT_TRUE(empty < def);
   EXPECT_FALSE(def < empty);
+  EXPECT_TRUE(c_null < def);
+  EXPECT_TRUE(c_empty < def);
+  EXPECT_FALSE(c_def < empty);
+  EXPECT_TRUE(empty < c_def);
+  EXPECT_FALSE(def < c_null);
+  EXPECT_FALSE(def < c_empty);
+  EXPECT_TRUE(empty < v_def);
+  EXPECT_FALSE(def < v_empty);
 
   EXPECT_TRUE(a < abc);
   EXPECT_FALSE(abc < a);
+  EXPECT_TRUE(c_a < abc);
+  EXPECT_FALSE(c_abc < a);
+  EXPECT_TRUE(a < c_abc);
+  EXPECT_FALSE(abc < c_a);
+  EXPECT_TRUE(a < v_abc);
+  EXPECT_FALSE(abc < v_a);
 
   EXPECT_TRUE(a < def);
   EXPECT_FALSE(def < a);
+  EXPECT_TRUE(c_a < def);
+  EXPECT_FALSE(c_def < a);
+  EXPECT_TRUE(a < c_def);
+  EXPECT_FALSE(def < c_a);
+  EXPECT_TRUE(a < v_def);
+  EXPECT_FALSE(def < v_a);
 
   EXPECT_TRUE(abc < def);
   EXPECT_FALSE(def < abc);
+  EXPECT_TRUE(c_abc < def);
+  EXPECT_FALSE(c_def < abc);
+  EXPECT_TRUE(abc < c_def);
+  EXPECT_FALSE(def < c_abc);
+  EXPECT_TRUE(abc < v_def);
+  EXPECT_FALSE(def < v_abc);
+
+  EXPECT_TRUE(a < ab);
+  EXPECT_TRUE(a < c_ab);
+  EXPECT_TRUE(a < v_ab);
+  EXPECT_TRUE(c_a < ab);
+  EXPECT_TRUE(c_a < v_ab);
+  EXPECT_TRUE(v_a < c_ab);
+  EXPECT_TRUE(v_a < v_ab);
 }
 
 TEST(WideString, OperatorEQ) {
@@ -152,8 +267,8 @@ TEST(WideString, OperatorEQ) {
   EXPECT_FALSE(wide_string_c2 == wide_string);
   EXPECT_FALSE(wide_string_c3 == wide_string);
 
-  const wchar_t* c_null_string = nullptr;
-  const wchar_t* c_empty_string = L"";
+  const wchar_t* const c_null_string = nullptr;
+  const wchar_t* const c_empty_string = L"";
   EXPECT_TRUE(null_string == c_null_string);
   EXPECT_TRUE(null_string == c_empty_string);
   EXPECT_TRUE(empty_string == c_null_string);
@@ -167,13 +282,13 @@ TEST(WideString, OperatorEQ) {
   EXPECT_TRUE(c_null_string == deleted_string);
   EXPECT_TRUE(c_empty_string == deleted_string);
 
-  const wchar_t* c_string_same1 = L"hello";
+  const wchar_t* const c_string_same1 = L"hello";
   EXPECT_TRUE(wide_string == c_string_same1);
   EXPECT_TRUE(c_string_same1 == wide_string);
 
-  const wchar_t* c_string1 = L"he";
-  const wchar_t* c_string2 = L"hellp";
-  const wchar_t* c_string3 = L"hellod";
+  const wchar_t* const c_string1 = L"he";
+  const wchar_t* const c_string2 = L"hellp";
+  const wchar_t* const c_string3 = L"hellod";
   EXPECT_FALSE(wide_string == c_string1);
   EXPECT_FALSE(wide_string == c_string2);
   EXPECT_FALSE(wide_string == c_string3);
@@ -253,8 +368,8 @@ TEST(WideString, OperatorNE) {
   EXPECT_TRUE(wide_string_c2 != wide_string);
   EXPECT_TRUE(wide_string_c3 != wide_string);
 
-  const wchar_t* c_null_string = nullptr;
-  const wchar_t* c_empty_string = L"";
+  const wchar_t* const c_null_string = nullptr;
+  const wchar_t* const c_empty_string = L"";
   EXPECT_FALSE(null_string != c_null_string);
   EXPECT_FALSE(null_string != c_empty_string);
   EXPECT_FALSE(empty_string != c_null_string);
@@ -268,13 +383,13 @@ TEST(WideString, OperatorNE) {
   EXPECT_FALSE(c_null_string != deleted_string);
   EXPECT_FALSE(c_empty_string != deleted_string);
 
-  const wchar_t* c_string_same1 = L"hello";
+  const wchar_t* const c_string_same1 = L"hello";
   EXPECT_FALSE(wide_string != c_string_same1);
   EXPECT_FALSE(c_string_same1 != wide_string);
 
-  const wchar_t* c_string1 = L"he";
-  const wchar_t* c_string2 = L"hellp";
-  const wchar_t* c_string3 = L"hellod";
+  const wchar_t* const c_string1 = L"he";
+  const wchar_t* const c_string2 = L"hellp";
+  const wchar_t* const c_string3 = L"hellod";
   EXPECT_TRUE(wide_string != c_string1);
   EXPECT_TRUE(wide_string != c_string2);
   EXPECT_TRUE(wide_string != c_string3);
@@ -283,6 +398,31 @@ TEST(WideString, OperatorNE) {
   EXPECT_TRUE(c_string3 != wide_string);
 }
 
+TEST(WideString, OperatorPlus) {
+  EXPECT_EQ(L"I like dogs", L"I like " + WideString(L"dogs"));
+  EXPECT_EQ(L"Dogs like me", WideString(L"Dogs") + L" like me");
+  EXPECT_EQ(L"Oh no, error number 42",
+            L"Oh no, error number " + WideString::Format(L"%d", 42));
+
+  {
+    // Make sure operator+= and Concat() increases string memory allocation
+    // geometrically.
+    int allocations = 0;
+    WideString str(L"ABCDEFGHIJKLMN");
+    const wchar_t* buffer = str.c_str();
+    for (size_t i = 0; i < 10000; ++i) {
+      str += L"!";
+      const wchar_t* new_buffer = str.c_str();
+      if (new_buffer != buffer) {
+        buffer = new_buffer;
+        ++allocations;
+      }
+    }
+    EXPECT_LT(allocations, 25);
+    EXPECT_GT(allocations, 10);
+  }
+}
+
 TEST(WideString, ConcatInPlace) {
   WideString fred;
   fred.Concat(L"FRED", 4);
@@ -510,7 +650,7 @@ TEST(WideString, Find) {
   EXPECT_FALSE(empty_string.Find(L'a').has_value());
   EXPECT_FALSE(empty_string.Find(L'\0').has_value());
 
-  pdfium::Optional<size_t> result;
+  Optional<size_t> result;
   WideString single_string(L"a");
   result = single_string.Find(L'a');
   ASSERT_TRUE(result.has_value());
@@ -563,63 +703,38 @@ TEST(WideString, UpperLower) {
   EXPECT_EQ(L"", empty);
 }
 
-TEST(WideString, TrimRight) {
+TEST(WideString, Trim) {
   WideString fred(L"  FRED  ");
-  fred.TrimRight();
-  EXPECT_EQ(L"  FRED", fred);
-  fred.TrimRight(L'E');
-  EXPECT_EQ(L"  FRED", fred);
-  fred.TrimRight(L'D');
-  EXPECT_EQ(L"  FRE", fred);
-  fred.TrimRight(L"ERP");
-  EXPECT_EQ(L"  F", fred);
+  fred.Trim();
+  EXPECT_EQ(L"FRED", fred);
+  fred.Trim(L'E');
+  EXPECT_EQ(L"FRED", fred);
+  fred.Trim(L'F');
+  EXPECT_EQ(L"RED", fred);
+  fred.Trim(L"ERP");
+  EXPECT_EQ(L"D", fred);
 
   WideString blank(L"   ");
-  blank.TrimRight(L"ERP");
+  blank.Trim(L"ERP");
   EXPECT_EQ(L"   ", blank);
-  blank.TrimRight(L'E');
+  blank.Trim(L'E');
   EXPECT_EQ(L"   ", blank);
-  blank.TrimRight();
+  blank.Trim();
   EXPECT_EQ(L"", blank);
 
   WideString empty;
-  empty.TrimRight(L"ERP");
+  empty.Trim(L"ERP");
   EXPECT_EQ(L"", empty);
-  empty.TrimRight(L'E');
+  empty.Trim(L'E');
   EXPECT_EQ(L"", empty);
-  empty.TrimRight();
+  empty.Trim();
   EXPECT_EQ(L"", empty);
-}
 
-TEST(WideString, TrimRightCopies) {
-  {
-    // With a single reference, no copy takes place.
-    WideString fred(L"  FRED  ");
-    const wchar_t* old_buffer = fred.c_str();
-    fred.TrimRight();
-    EXPECT_EQ(L"  FRED", fred);
-    EXPECT_EQ(old_buffer, fred.c_str());
-  }
-  {
-    // With multiple references, we must copy.
-    WideString fred(L"  FRED  ");
-    WideString other_fred = fred;
-    const wchar_t* old_buffer = fred.c_str();
-    fred.TrimRight();
-    EXPECT_EQ(L"  FRED", fred);
-    EXPECT_EQ(L"  FRED  ", other_fred);
-    EXPECT_NE(old_buffer, fred.c_str());
-  }
-  {
-    // With multiple references, but no modifications, no copy.
-    WideString fred(L"FRED");
-    WideString other_fred = fred;
-    const wchar_t* old_buffer = fred.c_str();
-    fred.TrimRight();
-    EXPECT_EQ(L"FRED", fred);
-    EXPECT_EQ(L"FRED", other_fred);
-    EXPECT_EQ(old_buffer, fred.c_str());
-  }
+  WideString abc(L"  ABCCBA  ");
+  abc.Trim(L"A");
+  EXPECT_EQ(L"  ABCCBA  ", abc);
+  abc.Trim(L" A");
+  EXPECT_EQ(L"BCCB", abc);
 }
 
 TEST(WideString, TrimLeft) {
@@ -681,6 +796,65 @@ TEST(WideString, TrimLeftCopies) {
   }
 }
 
+TEST(WideString, TrimRight) {
+  WideString fred(L"  FRED  ");
+  fred.TrimRight();
+  EXPECT_EQ(L"  FRED", fred);
+  fred.TrimRight(L'E');
+  EXPECT_EQ(L"  FRED", fred);
+  fred.TrimRight(L'D');
+  EXPECT_EQ(L"  FRE", fred);
+  fred.TrimRight(L"ERP");
+  EXPECT_EQ(L"  F", fred);
+
+  WideString blank(L"   ");
+  blank.TrimRight(L"ERP");
+  EXPECT_EQ(L"   ", blank);
+  blank.TrimRight(L'E');
+  EXPECT_EQ(L"   ", blank);
+  blank.TrimRight();
+  EXPECT_EQ(L"", blank);
+
+  WideString empty;
+  empty.TrimRight(L"ERP");
+  EXPECT_EQ(L"", empty);
+  empty.TrimRight(L'E');
+  EXPECT_EQ(L"", empty);
+  empty.TrimRight();
+  EXPECT_EQ(L"", empty);
+}
+
+TEST(WideString, TrimRightCopies) {
+  {
+    // With a single reference, no copy takes place.
+    WideString fred(L"  FRED  ");
+    const wchar_t* old_buffer = fred.c_str();
+    fred.TrimRight();
+    EXPECT_EQ(L"  FRED", fred);
+    EXPECT_EQ(old_buffer, fred.c_str());
+  }
+  {
+    // With multiple references, we must copy.
+    WideString fred(L"  FRED  ");
+    WideString other_fred = fred;
+    const wchar_t* old_buffer = fred.c_str();
+    fred.TrimRight();
+    EXPECT_EQ(L"  FRED", fred);
+    EXPECT_EQ(L"  FRED  ", other_fred);
+    EXPECT_NE(old_buffer, fred.c_str());
+  }
+  {
+    // With multiple references, but no modifications, no copy.
+    WideString fred(L"FRED");
+    WideString other_fred = fred;
+    const wchar_t* old_buffer = fred.c_str();
+    fred.TrimRight();
+    EXPECT_EQ(L"FRED", fred);
+    EXPECT_EQ(L"FRED", other_fred);
+    EXPECT_EQ(old_buffer, fred.c_str());
+  }
+}
+
 TEST(WideString, Reserve) {
   {
     WideString str;
@@ -703,20 +877,21 @@ TEST(WideString, Reserve) {
 }
 
 TEST(WideString, GetBuffer) {
+  WideString str1;
   {
-    WideString str;
-    wchar_t* buffer = str.GetBuffer(12);
-    wcscpy(buffer, L"clams");
-    str.ReleaseBuffer(str.GetStringLength());
-    EXPECT_EQ(L"clams", str);
+    pdfium::span<wchar_t> buffer = str1.GetBuffer(12);
+    wcscpy(buffer.data(), L"clams");
   }
+  str1.ReleaseBuffer(str1.GetStringLength());
+  EXPECT_EQ(L"clams", str1);
+
+  WideString str2(L"cl");
   {
-    WideString str(L"cl");
-    wchar_t* buffer = str.GetBuffer(12);
-    wcscpy(buffer + 2, L"ams");
-    str.ReleaseBuffer(str.GetStringLength());
-    EXPECT_EQ(L"clams", str);
+    pdfium::span<wchar_t> buffer = str2.GetBuffer(12);
+    wcscpy(buffer.data() + 2, L"ams");
   }
+  str2.ReleaseBuffer(str2.GetStringLength());
+  EXPECT_EQ(L"clams", str2);
 }
 
 TEST(WideString, ReleaseBuffer) {
@@ -825,11 +1000,11 @@ TEST(WideString, MultiCharReverseIterator) {
   EXPECT_TRUE(iter == multi_str.rbegin());
 }
 
-TEST(WideString, UTF16LE_Encode) {
+TEST(WideString, ToUTF16LE) {
   struct UTF16LEEncodeCase {
     WideString ws;
     ByteString bs;
-  } utf16le_encode_cases[] = {
+  } const utf16le_encode_cases[] = {
       {L"", ByteString("\0\0", 2)},
       {L"abc", ByteString("a\0b\0c\0\0\0", 8)},
       {L"abcdef", ByteString("a\0b\0c\0d\0e\0f\0\0\0", 14)},
@@ -840,11 +1015,144 @@ TEST(WideString, UTF16LE_Encode) {
 
   for (size_t i = 0; i < FX_ArraySize(utf16le_encode_cases); ++i) {
     EXPECT_EQ(utf16le_encode_cases[i].bs,
-              utf16le_encode_cases[i].ws.UTF16LE_Encode())
+              utf16le_encode_cases[i].ws.ToUTF16LE())
         << " for case number " << i;
   }
 }
 
+TEST(WideString, IsASCII) {
+  EXPECT_TRUE(WideString(L"xy\u007fz").IsASCII());
+  EXPECT_FALSE(WideString(L"xy\u0080z").IsASCII());
+  EXPECT_FALSE(WideString(L"xy\u2041z").IsASCII());
+}
+
+TEST(WideString, EqualsASCII) {
+  EXPECT_TRUE(WideString(L"").EqualsASCII(""));
+  EXPECT_FALSE(WideString(L"A").EqualsASCII(""));
+  EXPECT_FALSE(WideString(L"").EqualsASCII("A"));
+  EXPECT_FALSE(WideString(L"A").EqualsASCII("B"));
+  EXPECT_TRUE(WideString(L"ABC").EqualsASCII("ABC"));
+  EXPECT_FALSE(WideString(L"ABC").EqualsASCII("AEC"));
+  EXPECT_FALSE(WideString(L"\u00c1").EqualsASCII("\x41"));
+  EXPECT_FALSE(WideString(L"\u0141").EqualsASCII("\x41"));
+}
+
+TEST(WideString, EqualsASCIINoCase) {
+  EXPECT_TRUE(WideString(L"").EqualsASCIINoCase(""));
+  EXPECT_FALSE(WideString(L"A").EqualsASCIINoCase("b"));
+  EXPECT_TRUE(WideString(L"AbC").EqualsASCIINoCase("aBc"));
+  EXPECT_FALSE(WideString(L"ABc").EqualsASCIINoCase("AeC"));
+  EXPECT_FALSE(WideString(L"\u00c1").EqualsASCIINoCase("\x41"));
+  EXPECT_FALSE(WideString(L"\u0141").EqualsASCIINoCase("\x41"));
+}
+
+TEST(WideString, ToASCII) {
+  const char* kResult =
+      "x"
+      "\x02"
+      "\x7f"
+      "\x22"
+      "\x0c"
+      "y";
+  EXPECT_EQ(kResult, WideString(L"x"
+                                L"\u0082"
+                                L"\u00ff"
+                                L"\u0122"
+                                L"\u208c"
+                                L"y")
+                         .ToASCII());
+}
+
+TEST(WideString, ToLatin1) {
+  const char* kResult =
+      "x"
+      "\x82"
+      "\xff"
+      "\x22"
+      "\x8c"
+      "y";
+  EXPECT_EQ(kResult, WideString(L"x"
+                                L"\u0082"
+                                L"\u00ff"
+                                L"\u0122"
+                                L"\u208c"
+                                L"y")
+                         .ToLatin1());
+}
+
+TEST(WideString, ToDefANSI) {
+  EXPECT_EQ("", WideString().ToDefANSI());
+#if defined(OS_WIN)
+  const char* kResult =
+      "x"
+      "?"
+      "\xff"
+      "A"
+      "?"
+      "y";
+#else
+  const char* kResult =
+      "x"
+      "\x80"
+      "\xff"
+      "y";
+#endif
+  EXPECT_EQ(kResult, WideString(L"x"
+                                L"\u0080"
+                                L"\u00ff"
+                                L"\u0100"
+                                L"\u208c"
+                                L"y")
+                         .ToDefANSI());
+}
+
+TEST(WideString, FromASCII) {
+  EXPECT_EQ(L"", WideString::FromASCII(ByteStringView()));
+  const wchar_t* kResult =
+      L"x"
+      L"\u0002"
+      L"\u007f"
+      L"y";
+  EXPECT_EQ(kResult, WideString::FromASCII("x"
+                                           "\x82"
+                                           "\xff"
+                                           "y"));
+}
+
+TEST(WideString, FromLatin1) {
+  EXPECT_EQ(L"", WideString::FromLatin1(ByteStringView()));
+  const wchar_t* kResult =
+      L"x"
+      L"\u0082"
+      L"\u00ff"
+      L"y";
+  EXPECT_EQ(kResult, WideString::FromLatin1("x"
+                                            "\x82"
+                                            "\xff"
+                                            "y"));
+}
+
+TEST(WideString, FromDefANSI) {
+  EXPECT_EQ(L"", WideString::FromDefANSI(ByteStringView()));
+#if defined(OS_WIN)
+  const wchar_t* kResult =
+      L"x"
+      L"\u20ac"
+      L"\u00ff"
+      L"y";
+#else
+  const wchar_t* kResult =
+      L"x"
+      L"\u0080"
+      L"\u00ff"
+      L"y";
+#endif
+  EXPECT_EQ(kResult, WideString::FromDefANSI("x"
+                                             "\x80"
+                                             "\xff"
+                                             "y"));
+}
+
 TEST(WideStringView, FromVector) {
   std::vector<WideStringView::UnsignedType> null_vec;
   WideStringView null_string(null_vec);
@@ -879,29 +1187,59 @@ TEST(WideStringView, OperatorLT) {
   WideStringView a(L"a");
   WideStringView abc(L"\x0110qq");  // Comes InsertAtFront despite endianness.
   WideStringView def(L"\x1001qq");  // Comes InsertAtBack despite endianness.
+  const wchar_t* const c_null = nullptr;
+  const wchar_t* const c_empty = L"";
+  const wchar_t* const c_a = L"a";
+  const wchar_t* const c_abc = L"\x0110qq";
+  const wchar_t* const c_def = L"\x1001qq";
 
   EXPECT_FALSE(empty < empty);
   EXPECT_FALSE(a < a);
   EXPECT_FALSE(abc < abc);
   EXPECT_FALSE(def < def);
+  EXPECT_FALSE(c_null < empty);
+  EXPECT_FALSE(c_empty < empty);
+  EXPECT_FALSE(c_a < a);
+  EXPECT_FALSE(c_abc < abc);
+  EXPECT_FALSE(c_def < def);
+  EXPECT_FALSE(empty < c_null);
+  EXPECT_FALSE(empty < c_empty);
+  EXPECT_FALSE(a < c_a);
+  EXPECT_FALSE(abc < c_abc);
+  EXPECT_FALSE(def < c_def);
 
   EXPECT_TRUE(empty < a);
   EXPECT_FALSE(a < empty);
+  EXPECT_TRUE(empty < c_a);
+  EXPECT_FALSE(a < c_null);
+  EXPECT_FALSE(a < c_empty);
 
   EXPECT_TRUE(empty < abc);
   EXPECT_FALSE(abc < empty);
+  EXPECT_TRUE(empty < c_abc);
+  EXPECT_FALSE(abc < c_null);
+  EXPECT_FALSE(abc < c_empty);
 
   EXPECT_TRUE(empty < def);
   EXPECT_FALSE(def < empty);
+  EXPECT_TRUE(empty < c_def);
+  EXPECT_FALSE(def < c_null);
+  EXPECT_FALSE(def < c_empty);
 
   EXPECT_TRUE(a < abc);
   EXPECT_FALSE(abc < a);
+  EXPECT_TRUE(a < c_abc);
+  EXPECT_FALSE(abc < c_a);
 
   EXPECT_TRUE(a < def);
   EXPECT_FALSE(def < a);
+  EXPECT_TRUE(a < c_def);
+  EXPECT_FALSE(def < c_a);
 
   EXPECT_TRUE(abc < def);
   EXPECT_FALSE(def < abc);
+  EXPECT_TRUE(abc < c_def);
+  EXPECT_FALSE(def < c_abc);
 }
 
 TEST(WideStringView, OperatorEQ) {
@@ -940,13 +1278,13 @@ TEST(WideStringView, OperatorEQ) {
   EXPECT_FALSE(wide_string2 == wide_string_c);
   EXPECT_FALSE(wide_string3 == wide_string_c);
 
-  const wchar_t* c_string_same1 = L"hello";
+  const wchar_t* const c_string_same1 = L"hello";
   EXPECT_TRUE(wide_string_c == c_string_same1);
   EXPECT_TRUE(c_string_same1 == wide_string_c);
 
-  const wchar_t* c_string1 = L"he";
-  const wchar_t* c_string2 = L"hellp";
-  const wchar_t* c_string3 = L"hellod";
+  const wchar_t* const c_string1 = L"he";
+  const wchar_t* const c_string2 = L"hellp";
+  const wchar_t* const c_string3 = L"hellod";
   EXPECT_FALSE(wide_string_c == c_string1);
   EXPECT_FALSE(wide_string_c == c_string2);
   EXPECT_FALSE(wide_string_c == c_string3);
@@ -992,13 +1330,13 @@ TEST(WideStringView, OperatorNE) {
   EXPECT_TRUE(wide_string2 != wide_string_c);
   EXPECT_TRUE(wide_string3 != wide_string_c);
 
-  const wchar_t* c_string_same1 = L"hello";
+  const wchar_t* const c_string_same1 = L"hello";
   EXPECT_FALSE(wide_string_c != c_string_same1);
   EXPECT_FALSE(c_string_same1 != wide_string_c);
 
-  const wchar_t* c_string1 = L"he";
-  const wchar_t* c_string2 = L"hellp";
-  const wchar_t* c_string3 = L"hellod";
+  const wchar_t* const c_string1 = L"he";
+  const wchar_t* const c_string2 = L"hellp";
+  const wchar_t* const c_string3 = L"hellod";
   EXPECT_TRUE(wide_string_c != c_string1);
   EXPECT_TRUE(wide_string_c != c_string2);
   EXPECT_TRUE(wide_string_c != c_string3);
@@ -1017,7 +1355,7 @@ TEST(WideStringView, Find) {
   EXPECT_FALSE(empty_string.Find(L'a').has_value());
   EXPECT_FALSE(empty_string.Find(L'\0').has_value());
 
-  pdfium::Optional<size_t> result;
+  Optional<size_t> result;
   WideStringView single_string(L"a");
   result = single_string.Find(L'a');
   ASSERT_TRUE(result.has_value());
@@ -1183,92 +1521,55 @@ TEST(WideStringView, TrimmedRight) {
 }
 
 TEST(WideString, FormatWidth) {
-  {
-    WideString str;
-    str.Format(L"%5d", 1);
-    EXPECT_EQ(L"    1", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%d", 1);
-    EXPECT_EQ(L"1", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%*d", 5, 1);
-    EXPECT_EQ(L"    1", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%-1d", 1);
-    EXPECT_EQ(L"1", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%0d", 1);
-    EXPECT_EQ(L"1", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%1048576d", 1);
-    EXPECT_EQ(L"", str);
-  }
+  EXPECT_EQ(L"    1", WideString::Format(L"%5d", 1));
+  EXPECT_EQ(L"1", WideString::Format(L"%d", 1));
+  EXPECT_EQ(L"    1", WideString::Format(L"%*d", 5, 1));
+  EXPECT_EQ(L"1", WideString::Format(L"%-1d", 1));
+  EXPECT_EQ(L"1", WideString::Format(L"%0d", 1));
+  EXPECT_EQ(L"", WideString::Format(L"%1048576d", 1));
 }
 
 TEST(WideString, FormatPrecision) {
-  {
-    WideString str;
-    str.Format(L"%.2f", 1.12345);
-    EXPECT_EQ(L"1.12", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%.*f", 3, 1.12345);
-    EXPECT_EQ(L"1.123", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%f", 1.12345);
-    EXPECT_EQ(L"1.123450", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%-1f", 1.12345);
-    EXPECT_EQ(L"1.123450", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%0f", 1.12345);
-    EXPECT_EQ(L"1.123450", str);
-  }
-
-  {
-    WideString str;
-    str.Format(L"%.1048576f", 1.2);
-    EXPECT_EQ(L"", str);
-  }
+  EXPECT_EQ(L"1.12", WideString::Format(L"%.2f", 1.12345));
+  EXPECT_EQ(L"1.123", WideString::Format(L"%.*f", 3, 1.12345));
+  EXPECT_EQ(L"1.123450", WideString::Format(L"%f", 1.12345));
+  EXPECT_EQ(L"1.123450", WideString::Format(L"%-1f", 1.12345));
+  EXPECT_EQ(L"1.123450", WideString::Format(L"%0f", 1.12345));
+  EXPECT_EQ(L"", WideString::Format(L"%.1048576f", 1.2));
 }
 
 TEST(WideString, FormatOutOfRangeChar) {
-  WideString str;
-  str.Format(L"unsupported char '%c'", 0x00FF00FF);
+  EXPECT_NE(L"", WideString::Format(L"unsupported char '%c'", 0x00FF00FF));
+}
+
+TEST(WideString, FormatString) {
+  // %ls and wide characters are the reliable combination across platforms.
+  EXPECT_EQ(L"", WideString::Format(L"%ls", L""));
+  EXPECT_EQ(L"", WideString::Format(L"%ls", WideString().c_str()));
+  EXPECT_EQ(L"clams", WideString::Format(L"%ls", L"clams"));
+  EXPECT_EQ(L"cla", WideString::Format(L"%.3ls", L"clams"));
+  EXPECT_EQ(L"\u043e\u043f", WideString(L"\u043e\u043f"));
+
+#if !defined(OS_MACOSX)
+  // See https://bugs.chromium.org/p/pdfium/issues/detail?id=1132
+  EXPECT_EQ(L"\u043e\u043f", WideString::Format(L"\u043e\u043f"));
+  EXPECT_EQ(L"\u043e\u043f", WideString::Format(L"%ls", L"\u043e\u043f"));
+  EXPECT_EQ(L"\u043e", WideString::Format(L"%.1ls", L"\u043e\u043f"));
+#endif
 }
 
 TEST(WideString, Empty) {
   WideString empty_str;
   EXPECT_TRUE(empty_str.IsEmpty());
   EXPECT_EQ(0u, empty_str.GetLength());
+
   const wchar_t* cstr = empty_str.c_str();
+  EXPECT_NE(nullptr, cstr);
   EXPECT_EQ(0u, wcslen(cstr));
+
+  pdfium::span<const wchar_t> cspan = empty_str.AsSpan();
+  EXPECT_TRUE(cspan.empty());
+  EXPECT_EQ(nullptr, cspan.data());
 }
 
 TEST(CFX_WidString, InitializerList) {
@@ -1326,6 +1627,15 @@ TEST(WideString, MultiCharIterator) {
   EXPECT_EQ(static_cast<int32_t>(L'a' + L'b' + L'c'), sum);
 }
 
+TEST(WideString, StdBegin) {
+  WideString one_str(L"abc");
+  std::vector<wchar_t> vec(std::begin(one_str), std::end(one_str));
+  ASSERT_EQ(3u, vec.size());
+  EXPECT_EQ(L'a', vec[0]);
+  EXPECT_EQ(L'b', vec[1]);
+  EXPECT_EQ(L'c', vec[2]);
+}
+
 TEST(WideString, AnyAllNoneOf) {
   WideString str(L"aaaaaaaaaaaaaaaaab");
   EXPECT_FALSE(std::all_of(str.begin(), str.end(),
@@ -1606,4 +1916,15 @@ TEST(WideStringView, WideOStreamOverload) {
   }
 }
 
+TEST(WideString, FX_HashCode_Wide) {
+  EXPECT_EQ(0u, FX_HashCode_GetW(L"", false));
+  EXPECT_EQ(65u, FX_HashCode_GetW(L"A", false));
+  EXPECT_EQ(97u, FX_HashCode_GetW(L"A", true));
+  EXPECT_EQ(1313 * 65u + 66u, FX_HashCode_GetW(L"AB", false));
+  EXPECT_EQ(FX_HashCode_GetAsIfW("AB\xff", false),
+            FX_HashCode_GetW(L"AB\xff", false));
+  EXPECT_EQ(FX_HashCode_GetAsIfW("AB\xff", true),
+            FX_HashCode_GetW(L"AB\xff", true));
+}
+
 }  // namespace fxcrt
