@@ -9,12 +9,12 @@
 #include <algorithm>
 #include <utility>
 
+#include "build/build_config.h"
 #include "core/fxcrt/fx_extension.h"
 #include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
 #include "xfa/fwl/cfwl_app.h"
 #include "xfa/fwl/cfwl_eventtarget.h"
-#include "xfa/fwl/cfwl_form.h"
 #include "xfa/fwl/cfwl_messagekey.h"
 #include "xfa/fwl/cfwl_messagekillfocus.h"
 #include "xfa/fwl/cfwl_messagemouse.h"
@@ -25,7 +25,6 @@
 
 CFWL_NoteDriver::CFWL_NoteDriver()
     : m_pNoteLoop(pdfium::MakeUnique<CFWL_NoteLoop>()) {
-  PushNoteLoop(m_pNoteLoop.get());
 }
 
 CFWL_NoteDriver::~CFWL_NoteDriver() = default;
@@ -62,19 +61,6 @@ void CFWL_NoteDriver::UnregisterEventTarget(CFWL_Widget* pListener) {
     it->second->FlagInvalid();
 }
 
-void CFWL_NoteDriver::PushNoteLoop(CFWL_NoteLoop* pNoteLoop) {
-  m_NoteLoopQueue.emplace_back(pNoteLoop);
-}
-
-CFWL_NoteLoop* CFWL_NoteDriver::PopNoteLoop() {
-  if (m_NoteLoopQueue.empty())
-    return nullptr;
-
-  CFWL_NoteLoop* p = m_NoteLoopQueue.back().Get();
-  m_NoteLoopQueue.pop_back();
-  return p;
-}
-
 bool CFWL_NoteDriver::SetFocus(CFWL_Widget* pFocus) {
   if (m_pFocus == pFocus)
     return true;
@@ -88,12 +74,6 @@ bool CFWL_NoteDriver::SetFocus(CFWL_Widget* pFocus) {
     }
   }
   if (pFocus) {
-    CFWL_Widget* pWidget =
-        pFocus->GetOwnerApp()->GetWidgetMgr()->GetSystemFormWidget(pFocus);
-    CFWL_Form* pForm = static_cast<CFWL_Form*>(pWidget);
-    if (pForm)
-      pForm->SetSubFocus(pFocus);
-
     if (IFWL_WidgetDelegate* pDelegate = pFocus->GetDelegate()) {
       CFWL_MessageSetFocus ms(nullptr, pFocus);
       pDelegate->OnProcessMessage(&ms);
@@ -103,14 +83,14 @@ bool CFWL_NoteDriver::SetFocus(CFWL_Widget* pFocus) {
 }
 
 void CFWL_NoteDriver::Run() {
-#if _FX_OS_ == _FX_OS_LINUX_ || _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
+#if defined(OS_LINUX) || defined(OS_WIN)
   for (;;) {
     CFWL_NoteLoop* pTopLoop = GetTopLoop();
     if (!pTopLoop || !pTopLoop->ContinueModal())
       break;
     UnqueueMessageAndProcess(pTopLoop);
   }
-#endif  // _FX_OS_ == _FX_OS_LINUX_ || _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
+#endif  // defined(OS_LINUX) || defined(OS_WIN)
 }
 
 void CFWL_NoteDriver::NotifyTargetHide(CFWL_Widget* pNoteTarget) {
@@ -131,34 +111,6 @@ void CFWL_NoteDriver::NotifyTargetDestroy(CFWL_Widget* pNoteTarget) {
     m_pGrab = nullptr;
 
   UnregisterEventTarget(pNoteTarget);
-
-  for (const auto& pWidget : m_Forms) {
-    CFWL_Form* pForm = static_cast<CFWL_Form*>(pWidget.Get());
-    if (!pForm)
-      continue;
-
-    CFWL_Widget* pSubFocus = pForm->GetSubFocus();
-    if (!pSubFocus)
-      return;
-
-    if (pSubFocus == pNoteTarget)
-      pForm->SetSubFocus(nullptr);
-  }
-}
-
-void CFWL_NoteDriver::RegisterForm(CFWL_Widget* pForm) {
-  if (!pForm || pdfium::ContainsValue(m_Forms, pForm))
-    return;
-
-  m_Forms.emplace_back(pForm);
-  if (m_Forms.size() == 1 && !m_NoteLoopQueue.empty() && m_NoteLoopQueue[0])
-    m_NoteLoopQueue[0]->SetMainForm(pForm);
-}
-
-void CFWL_NoteDriver::UnRegisterForm(CFWL_Widget* pForm) {
-  auto iter = std::find(m_Forms.begin(), m_Forms.end(), pForm);
-  if (iter != m_Forms.end())
-    m_Forms.erase(iter);
 }
 
 void CFWL_NoteDriver::QueueMessage(std::unique_ptr<CFWL_Message> pMessage) {
@@ -177,10 +129,6 @@ void CFWL_NoteDriver::UnqueueMessageAndProcess(CFWL_NoteLoop* pNoteLoop) {
   ProcessMessage(std::move(pMessage));
 }
 
-CFWL_NoteLoop* CFWL_NoteDriver::GetTopLoop() const {
-  return !m_NoteLoopQueue.empty() ? m_NoteLoopQueue.back().Get() : nullptr;
-}
-
 void CFWL_NoteDriver::ProcessMessage(std::unique_ptr<CFWL_Message> pMessage) {
   CFWL_Widget* pMessageForm = pMessage->GetDstTarget();
   if (!pMessageForm)
@@ -246,7 +194,7 @@ bool CFWL_NoteDriver::DoKillFocus(CFWL_Message* pMessage,
 
 bool CFWL_NoteDriver::DoKey(CFWL_Message* pMessage, CFWL_Widget* pMessageForm) {
   CFWL_MessageKey* pMsg = static_cast<CFWL_MessageKey*>(pMessage);
-#if (_FX_OS_ != _FX_OS_MACOSX_)
+#if !defined(OS_MACOSX)
   if (pMsg->m_dwCmd == FWL_KeyCommand::KeyDown &&
       pMsg->m_dwKeyCode == FWL_VKEY_Tab) {
     CFWL_WidgetMgr* pWidgetMgr = pMessageForm->GetOwnerApp()->GetWidgetMgr();
@@ -303,9 +251,6 @@ bool CFWL_NoteDriver::DoMouse(CFWL_Message* pMessage,
 bool CFWL_NoteDriver::DoWheel(CFWL_Message* pMessage,
                               CFWL_Widget* pMessageForm) {
   CFWL_WidgetMgr* pWidgetMgr = pMessageForm->GetOwnerApp()->GetWidgetMgr();
-  if (!pWidgetMgr)
-    return false;
-
   CFWL_MessageMouseWheel* pMsg = static_cast<CFWL_MessageMouseWheel*>(pMessage);
   CFWL_Widget* pDst = pWidgetMgr->GetWidgetAtPoint(pMessageForm, pMsg->m_pos);
   if (!pDst)
@@ -319,8 +264,6 @@ bool CFWL_NoteDriver::DoWheel(CFWL_Message* pMessage,
 bool CFWL_NoteDriver::DoMouseEx(CFWL_Message* pMessage,
                                 CFWL_Widget* pMessageForm) {
   CFWL_WidgetMgr* pWidgetMgr = pMessageForm->GetOwnerApp()->GetWidgetMgr();
-  if (!pWidgetMgr)
-    return false;
   CFWL_Widget* pTarget = nullptr;
   if (m_pGrab)
     pTarget = m_pGrab.Get();
@@ -364,34 +307,14 @@ void CFWL_NoteDriver::MouseSecondary(CFWL_Message* pMessage) {
 }
 
 bool CFWL_NoteDriver::IsValidMessage(CFWL_Message* pMessage) {
-  for (const auto& pNoteLoop : m_NoteLoopQueue) {
-    CFWL_Widget* pForm = pNoteLoop->GetForm();
-    if (pForm && pForm == pMessage->GetDstTarget())
-      return true;
-  }
-  for (const auto& pWidget : m_Forms) {
-    auto* pForm = static_cast<const CFWL_Form*>(pWidget.Get());
-    if (pForm == pMessage->GetDstTarget())
-      return true;
-  }
-  return false;
+  CFWL_Widget* pForm = m_pNoteLoop->GetForm();
+  return pForm && pForm == pMessage->GetDstTarget();
 }
 
 CFWL_Widget* CFWL_NoteDriver::GetMessageForm(CFWL_Widget* pDstTarget) {
-  if (m_NoteLoopQueue.empty())
-    return nullptr;
-
-  CFWL_Widget* pMessageForm = nullptr;
-  if (m_NoteLoopQueue.size() > 1)
-    pMessageForm = m_NoteLoopQueue.back()->GetForm();
-  else if (!pdfium::ContainsValue(m_Forms, pDstTarget))
-    pMessageForm = pDstTarget;
-
+  CFWL_Widget* pMessageForm = m_pNoteLoop->GetForm();
   if (!pMessageForm && pDstTarget) {
     CFWL_WidgetMgr* pWidgetMgr = pDstTarget->GetOwnerApp()->GetWidgetMgr();
-    if (!pWidgetMgr)
-      return nullptr;
-
     pMessageForm = pWidgetMgr->GetSystemFormWidget(pDstTarget);
   }
   return pMessageForm;
