@@ -12,9 +12,10 @@
 #include "core/fxcrt/fx_string.h"
 #include "core/fxge/cfx_defaultrenderdevice.h"
 #include "core/fxge/cfx_renderdevice.h"
+#include "core/fxge/dib/cfx_dibitmap.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "testing/test_support.h"
 #include "testing/utils/bitmap_saver.h"
+#include "testing/utils/hash.h"
 #include "third_party/base/ptr_util.h"
 
 class BarcodeTest : public testing::Test {
@@ -40,14 +41,10 @@ class BarcodeTest : public testing::Test {
 
   CFX_Barcode* barcode() const { return barcode_.get(); }
 
-  bool Create(BC_TYPE type) {
+  void Create(BC_TYPE type) {
     barcode_ = CFX_Barcode::Create(type);
-    if (!barcode_)
-      return false;
-
     barcode_->SetHeight(298);
     barcode_->SetWidth(418);
-    return true;
   }
 
   bool RenderDevice() {
@@ -72,77 +69,102 @@ class BarcodeTest : public testing::Test {
 };
 
 TEST_F(BarcodeTest, Code39) {
-  EXPECT_TRUE(Create(BC_CODE39));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+  Create(BC_CODE39);
+  EXPECT_TRUE(barcode()->Encode(L"CLAMS"));
   RenderDevice();
   EXPECT_EQ("cd4cd3f36da38ff58d9f621827018903", BitmapChecksum());
 }
 
 TEST_F(BarcodeTest, CodaBar) {
-  EXPECT_TRUE(Create(BC_CODABAR));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+  Create(BC_CODABAR);
+  EXPECT_TRUE(barcode()->Encode(L"$123-456"));
   RenderDevice();
-  EXPECT_EQ("481189dc4f86eddb8c42343c9b8ef1dd", BitmapChecksum());
+  EXPECT_EQ("5fad4fc19f099001a0fe83c89430c977", BitmapChecksum());
+}
+
+TEST_F(BarcodeTest, DISABLED_CodaBarLetters) {
+  Create(BC_CODABAR);
+  EXPECT_FALSE(barcode()->Encode(L"clams"));
 }
 
 TEST_F(BarcodeTest, Code128) {
-  EXPECT_TRUE(Create(BC_CODE128));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+  Create(BC_CODE128);
+  EXPECT_TRUE(barcode()->Encode(L"Clams"));
   RenderDevice();
-  EXPECT_EQ("11b21c178a9fd866d8be196c2103b263", BitmapChecksum());
+  EXPECT_EQ("6351f0f6e997050e4658bbb4777aef74", BitmapChecksum());
 }
 
-TEST_F(BarcodeTest, Code128_B) {
-  EXPECT_TRUE(Create(BC_CODE128_B));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+TEST_F(BarcodeTest, Code128B) {
+  Create(BC_CODE128_B);
+  EXPECT_TRUE(barcode()->Encode(L"Clams"));
   RenderDevice();
-  EXPECT_EQ("11b21c178a9fd866d8be196c2103b263", BitmapChecksum());
+  EXPECT_EQ("6351f0f6e997050e4658bbb4777aef74", BitmapChecksum());
 }
 
-TEST_F(BarcodeTest, Code128_C) {
-  EXPECT_TRUE(Create(BC_CODE128_C));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+TEST_F(BarcodeTest, Code128C) {
+  Create(BC_CODE128_C);
+  EXPECT_TRUE(barcode()->Encode(L"123456"));
   RenderDevice();
-  EXPECT_EQ("6284ec8503d5a948c9518108da33cdd3", BitmapChecksum());
+  EXPECT_EQ("fba730a807ba6363f9bd2bc7f8c56d1f", BitmapChecksum());
+}
+
+TEST_F(BarcodeTest, DISABLED_Code128CLetters) {
+  Create(BC_CODE128_C);
+  EXPECT_FALSE(barcode()->Encode(L"clams"));
 }
 
 TEST_F(BarcodeTest, Ean8) {
-  EXPECT_TRUE(Create(BC_EAN8));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+  Create(BC_EAN8);
+  EXPECT_TRUE(barcode()->Encode(L"123456"));
   RenderDevice();
-  EXPECT_EQ("22d85bcb02d48f48813f02a1cc9cfe8c", BitmapChecksum());
+  EXPECT_EQ("aff88491ac46ca6217d780d185300cde", BitmapChecksum());
+}
+
+TEST_F(BarcodeTest, DISABLED_Ean8Letters) {
+  Create(BC_EAN8);
+  EXPECT_FALSE(barcode()->Encode(L"clams"));
 }
 
 TEST_F(BarcodeTest, UPCA) {
-  EXPECT_TRUE(Create(BC_UPCA));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+  Create(BC_UPCA);
+  EXPECT_TRUE(barcode()->Encode(L"123456"));
   RenderDevice();
-  EXPECT_EQ("cce41fc30852744c44b3353059b568b4", BitmapChecksum());
+  EXPECT_EQ("fe26a5714cff7ffe3f9b02183efc435b", BitmapChecksum());
+}
+
+TEST_F(BarcodeTest, DISABLED_UPCALetters) {
+  Create(BC_UPCA);
+  EXPECT_FALSE(barcode()->Encode(L"clams"));
 }
 
 TEST_F(BarcodeTest, Ean13) {
-  EXPECT_TRUE(Create(BC_EAN13));
-  EXPECT_TRUE(barcode()->Encode(L"clams"));
+  Create(BC_EAN13);
+  EXPECT_TRUE(barcode()->Encode(L"123456"));
   RenderDevice();
-  EXPECT_EQ("187091ec1fd1830fc4d41d40a923d4fb", BitmapChecksum());
+  EXPECT_EQ("72d2190b98d635c32834bf67552e561e", BitmapChecksum());
+}
+
+TEST_F(BarcodeTest, DISABLED_Ean13Letters) {
+  Create(BC_EAN13);
+  EXPECT_FALSE(barcode()->Encode(L"clams"));
 }
 
 TEST_F(BarcodeTest, Pdf417) {
-  EXPECT_TRUE(Create(BC_PDF417));
+  Create(BC_PDF417);
   EXPECT_TRUE(barcode()->Encode(L"clams"));
   RenderDevice();
   EXPECT_EQ("191e35d11613901b7d5d51033689aa89", BitmapChecksum());
 }
 
 TEST_F(BarcodeTest, DataMatrix) {
-  EXPECT_TRUE(Create(BC_DATAMATRIX));
+  Create(BC_DATAMATRIX);
   EXPECT_TRUE(barcode()->Encode(L"clams"));
   RenderDevice();
   EXPECT_EQ("5e5cd9a680b86fcd4ffd53ed36e3c980", BitmapChecksum());
 }
 
 TEST_F(BarcodeTest, QrCode) {
-  EXPECT_TRUE(Create(BC_QR_CODE));
+  Create(BC_QR_CODE);
   EXPECT_TRUE(barcode()->Encode(L"clams"));
   RenderDevice();
   EXPECT_EQ("4751c6e0f67749fabe24f787128decee", BitmapChecksum());
