@@ -15,18 +15,14 @@
 CFWL_CaretTP::CFWL_CaretTP() {}
 CFWL_CaretTP::~CFWL_CaretTP() {}
 
-void CFWL_CaretTP::DrawBackground(CFWL_ThemeBackground* pParams) {
-  if (!pParams)
-    return;
-
-  switch (pParams->m_iPart) {
+void CFWL_CaretTP::DrawBackground(const CFWL_ThemeBackground& pParams) {
+  switch (pParams.m_iPart) {
     case CFWL_Part::Background: {
-      if (!(pParams->m_dwStates & CFWL_PartState_HightLight))
+      if (!(pParams.m_dwStates & CFWL_PartState_HightLight))
         return;
 
-      DrawCaretBK(
-          pParams->m_pGraphics.Get(), pParams->m_dwStates, &pParams->m_rtPart,
-          static_cast<CXFA_GEColor*>(pParams->m_pData), &pParams->m_matrix);
+      DrawCaretBK(pParams.m_pGraphics.Get(), pParams.m_dwStates,
+                  pParams.m_rtPart, pParams.m_matrix);
       break;
     }
     default:
@@ -36,16 +32,10 @@ void CFWL_CaretTP::DrawBackground(CFWL_ThemeBackground* pParams) {
 
 void CFWL_CaretTP::DrawCaretBK(CXFA_Graphics* pGraphics,
                                uint32_t dwStates,
-                               const CFX_RectF* pRect,
-                               CXFA_GEColor* crFill,
-                               CFX_Matrix* pMatrix) {
+                               const CFX_RectF& rect,
+                               const CFX_Matrix& matrix) {
   CXFA_GEPath path;
-  CFX_RectF rect = *pRect;
   path.AddRectangle(rect.left, rect.top, rect.width, rect.height);
-  if (crFill) {
-    pGraphics->SetFillColor(*crFill);
-  } else {
-    pGraphics->SetFillColor(CXFA_GEColor(ArgbEncode(255, 0, 0, 0)));
-  }
-  pGraphics->FillPath(&path, FXFILL_WINDING, pMatrix);
+  pGraphics->SetFillColor(CXFA_GEColor(ArgbEncode(255, 0, 0, 0)));
+  pGraphics->FillPath(&path, FXFILL_WINDING, &matrix);
 }
