@@ -15,7 +15,7 @@
 #include "core/fpdfapi/parser/cpdf_document.h"
 #include "core/fxge/cfx_fontmgr.h"
 #include "core/fxge/cfx_gemodule.h"
-#include "third_party/base/ptr_util.h"
+#include "xfa/fgas/font/cfgas_defaultfontmanager.h"
 #include "xfa/fgas/font/cfgas_gefont.h"
 #include "xfa/fgas/font/fgas_fontutils.h"
 #include "xfa/fxfa/cxfa_ffapp.h"
@@ -25,10 +25,9 @@ CXFA_FontMgr::CXFA_FontMgr() = default;
 
 CXFA_FontMgr::~CXFA_FontMgr() = default;
 
-RetainPtr<CFGAS_GEFont> CXFA_FontMgr::GetFont(
-    CXFA_FFDoc* hDoc,
-    const WideStringView& wsFontFamily,
-    uint32_t dwFontStyles) {
+RetainPtr<CFGAS_GEFont> CXFA_FontMgr::GetFont(CXFA_FFDoc* hDoc,
+                                              WideStringView wsFontFamily,
+                                              uint32_t dwFontStyles) {
   uint32_t dwHash = FX_HashCode_GetW(wsFontFamily, false);
   ByteString bsKey = ByteString::Format("%u%u%u", dwHash, dwFontStyles, 0xFFFF);
   auto iter = m_FontMap.find(bsKey);
@@ -44,9 +43,10 @@ RetainPtr<CFGAS_GEFont> CXFA_FontMgr::GetFont(
     if (pFont)
       return pFont;
   }
-  if (!pFont)
-    pFont = m_pDefFontMgr.GetFont(hDoc->GetApp()->GetFDEFontMgr(), wsFontFamily,
-                                  dwFontStyles);
+  if (!pFont) {
+    pFont = CFGAS_DefaultFontManager::GetFont(hDoc->GetApp()->GetFDEFontMgr(),
+                                              wsFontFamily, dwFontStyles);
+  }
 
   if (!pFont && pMgr) {
     pFont = pMgr->GetFont(wsEnglishName.AsStringView(), dwFontStyles, false);
@@ -55,8 +55,8 @@ RetainPtr<CFGAS_GEFont> CXFA_FontMgr::GetFont(
   }
 
   if (!pFont) {
-    pFont = m_pDefFontMgr.GetDefaultFont(hDoc->GetApp()->GetFDEFontMgr(),
-                                         wsFontFamily, dwFontStyles);
+    pFont = CFGAS_DefaultFontManager::GetDefaultFont(
+        hDoc->GetApp()->GetFDEFontMgr(), wsFontFamily, dwFontStyles);
   }
 
   if (!pFont) {
